/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "extIApplication.idl"

/**
 * The core STEEL object, available in the global scope
 */
[scriptable, uuid(a4cb1fcd-6af7-4251-9879-28fd120b1c9d)]
interface steelIApplication : extIApplication {
  /**
   * Returns true if the application is running on a Mac OS X. This is intended
   * for situations where Mac does things differently, e.g. The application
   * doesn't support print preview on Mac because Mac provides a preview option
   * in the print dialog.
   */
  readonly attribute boolean platformIsMac;
};
