/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIShellService.idl"

[scriptable, uuid(c6bcecde-6f2e-417c-bc30-466f560c3b51)]
interface nsIWindowsShellService : nsIShellService
{
  /**
   * Perform some Windows 7+ shortcut maintenance. This is called at program startup.
   */
  void shortcutMaintenance();
};
